#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>

#include <stdatomic.h>
#include <ctype.h>

#define checkError(expr,userMsg)\
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        }\
    } while(0)
#define checkErrorPthread(err,userMsg) \
    do {\
        int _err = err;\
        if (_err > 0) { \
            errno = _err;\
            checkError(0,userMsg);\
        } \
    } while(0);

typedef struct {
    char* fPath;
    char c;
} inData_t;

typedef struct {
    char c;
    int n;
} outData_t;

int globalCnt = 0;
int globalMax = -1;
char maxC = -1;
pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg) {

    inData_t* data = (inData_t*)arg;
    FILE* f = fopen(data->fPath, "r");
    checkError(f != NULL, "fopen");

    char c = 0;
    int cnt = 0;
    while ((c = fgetc(f)) != EOF) {
        if ('a' <= data->c && data->c <= 'z' && tolower(c) == data->c) {
            cnt++;
        }
        else if (data->c == ('z' + 1) && 
            !('a' <= tolower(c) && tolower(c) <= 'z')){
            cnt++;
        }
    }
    printf("%c: %d\n", data->c,  cnt);
    fclose(f);

    checkErrorPthread(pthread_mutex_lock(&lock), "lock");
    globalCnt += cnt;
    if (cnt > globalMax) {
        globalMax = cnt;
        maxC = data->c;
    }
    checkErrorPthread(pthread_mutex_unlock(&lock), "unlock");

    return NULL;
}

int main(int argc, char** argv) {

    checkError(argc == 2, "args");

    int n = 27;
    pthread_t* tids = malloc(n*sizeof(pthread_t));
    checkError(tids != NULL, "malloc");

    inData_t* args = malloc(n*sizeof(inData_t));
    checkError(args != NULL, "malloc");

    for (int i = 0; i < n; i++) {
        args[i].fPath = argv[1];
        args[i].c = 'a' + i;
        checkErrorPthread(
            pthread_create(&tids[i], NULL, threadFunc, &args[i]),
            "create"
        );
    }

    for (int i = 0; i < n; i++) {
        checkErrorPthread(
            pthread_join(tids[i], NULL),
            "join"
        );
    }

    printf("Ukupno karaktera: %d\n", globalCnt);
    printf("MaxC: %c\nBroj: %d\n", maxC == ('z' + 1) ? '.' : maxC, globalMax);

    free(tids);
    free(args);

    exit(EXIT_SUCCESS);
}