#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>

#include <stdatomic.h>

#define checkError(expr,userMsg)\
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        }\
    } while(0)
#define checkErrorPthread(err,userMsg) \
    do {\
        int _err = err;\
        if (_err > 0) { \
            errno = _err;\
            checkError(0,userMsg);\
        } \
    } while(0);

typedef struct {
    char* fPath;
} inData_t;

int globalCnt = 0;
pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;

void* threadFunc(void* arg) {

    inData_t* data = (inData_t*)arg;
    FILE* f = fopen(data->fPath, "r");
    checkError(f != NULL, "fopen");

    int cnt = 0;
    while (fgetc(f) != EOF) {
        cnt++;
    }

    fclose(f);

    checkErrorPthread(pthread_mutex_lock(&globalLock), "lock");
    globalCnt += cnt;
    checkErrorPthread(pthread_mutex_unlock(&globalLock), "unlock");

    return NULL;
}

int main(int argc, char** argv) {

    checkError(argc > 1, "args");

    int n = argc-1;
    pthread_t* tids = malloc(n*sizeof(pthread_t));
    checkError(tids != NULL, "malloc");

    inData_t* args = malloc(n*sizeof(inData_t));
    checkError(args != NULL, "args");

    atomic_init(&globalCnt, 0);

    for (int i = 0; i < n; i++) {
        args[i].fPath = argv[i+1];
        checkErrorPthread(
            pthread_create(&tids[i], NULL, threadFunc, &args[i]),
            "create"
        );
    }

    for (int i = 0; i < n; i++) {
        checkErrorPthread(pthread_join(tids[i], NULL), "join");
    }

    printf("Ukupno karaktera: %d\n", atomic_load(&globalCnt));

    free(tids);
    free(args);

    exit(EXIT_SUCCESS);
}